package;

import managers.ConfigManager;
import managers.SoundManager;
import classes.AtlasSprite;
import classes.BuildRoomFrame;
import classes.Cannon;
import classes.DeleteRoomFrame;
import classes.Explosion;
import classes.Hud;
import classes.Invader;
import classes.Rocket;
import classes.Room;
import classes.Worker;
import enums.GameState;
import enums.RoomType;
import flixel.FlxG;
import flixel.FlxState;
import flixel.group.FlxSpriteGroup;
import flixel.text.FlxText;
import flixel.util.FlxColor;
import helpers.FpsCalculator;
import managers.CannonManager;
import managers.CursorManager;
import managers.EventManager;
import managers.ExplosionManager;
import managers.InvaderManager;
import managers.MapManager;
import managers.RocketManager;
import managers.RoomManager;
import managers.StateManager;
import managers.WarehouseManager;
import managers.WorkerManager;
import windows.AboutWindow;
import windows.BuildWindow;
import windows.PlaceWindow;
import windows.RemoveWindow;
import windows.WelcomeWindow;
import windows.WinWindow;

class PlayState extends FlxState
{
	public static var instance(default, null):PlayState = null; // Simple singleton access thing

	var background:AtlasSprite;
	var buildGrid:AtlasSprite;
	var gameHud:Hud;
	var buildRoomFrame:BuildRoomFrame;
	var deleteRoomFrame:DeleteRoomFrame;

	var text_debug:FlxText;

	var Window_Layer:FlxSpriteGroup;
	var aboutWindow:AboutWindow;
	var buildWindow:BuildWindow;
	var placeWindow:PlaceWindow;
	var removeWindow:RemoveWindow;
	var welcomeWindow:WelcomeWindow;
	var winWindow:WinWindow;

	var Invader_Layer:FlxTypedSpriteGroup<Invader>;
	var Cannon_Layer:FlxTypedSpriteGroup<Cannon>;
	var Rocket_Layer:FlxTypedSpriteGroup<Rocket>;
	var Room_Layer:FlxTypedSpriteGroup<Room>;
	var Explosion_Layer:FlxTypedSpriteGroup<Explosion>;
	var Worker_Layer:FlxTypedSpriteGroup<Worker>;

	override public function create()
	{
		super.create();

		instance = this;

		SoundManager.init();
		CursorManager.setCursorType(Normal);

		initBgGraphic();
		initExplosions();
		initRooms();
		initWorkers();
		initInvaders();
		initCannon();
		initRockets();
		initHud();
		initWindows();
		initDebug();

		EventManager.subscribe(GameStateInit, gameStateInit);
		EventManager.subscribe(GameStateExit, gameStateExit);
		EventManager.subscribe(AllInvadersRemoved, allInvadersRemoved);

		StateManager.setGameState(Welcome);
	}

	override public function update(elapsed:Float)
	{
		super.update(elapsed);
		WarehouseManager.addElapsed(elapsed);
		updateDebugInfoLine(elapsed);
	}

	function initBgGraphic()
	{
		background = new AtlasSprite();
		background.assignImageFromAtlas("SIE_bg");
		background.setPosition(0, 0);
		add(background);

		buildGrid = new AtlasSprite();
		buildGrid.assignImageFromAtlas("build_grid");
		buildGrid.setPosition(40, 440);
		buildGrid.visible = false;
		add(buildGrid);
	}

	function initHud()
	{
		gameHud = new Hud();
		add(gameHud);
	}

	function initRooms()
	{
		Room_Layer = new FlxTypedSpriteGroup<Room>();
		RoomManager.init(Room_Layer);
		add(Room_Layer);
	}

	function initWorkers()
	{
		Worker_Layer = new FlxTypedSpriteGroup<Worker>();
		WorkerManager.init(Worker_Layer);
		add(Worker_Layer);
	}

	function initCannon()
	{
		Rocket_Layer = new FlxTypedSpriteGroup<Rocket>();
		RocketManager.init(Rocket_Layer);
		add(Rocket_Layer);
	}

	function initRockets()
	{
		Cannon_Layer = new FlxTypedSpriteGroup<Cannon>();
		CannonManager.init(Cannon_Layer);
		add(Cannon_Layer);
	}

	function initInvaders()
	{
		Invader_Layer = new FlxTypedSpriteGroup<Invader>();
		InvaderManager.init(Invader_Layer);
		add(Invader_Layer);
	}

	function initExplosions()
	{
		Explosion_Layer = new FlxTypedSpriteGroup<Explosion>();
		ExplosionManager.init(Explosion_Layer);
		add(Explosion_Layer);
	}

	function initWindows()
	{
		Window_Layer = new FlxSpriteGroup();

		aboutWindow = new AboutWindow();
		Window_Layer.add(aboutWindow);

		buildWindow = new BuildWindow();
		Window_Layer.add(buildWindow);

		placeWindow = new PlaceWindow();
		Window_Layer.add(placeWindow);

		removeWindow = new RemoveWindow();
		Window_Layer.add(removeWindow);

		welcomeWindow = new WelcomeWindow();
		Window_Layer.add(welcomeWindow);

		winWindow = new WinWindow();
		Window_Layer.add(winWindow);

		add(Window_Layer);
	}

	function initDebug()
	{
		text_debug = new FlxText(5, 770, 470, "", 14);
		text_debug.alignment = CENTER;
		text_debug.color = FlxColor.WHITE;
		add(text_debug);
	}

	function enableHud(enable:Bool)
	{
		gameHud.active = enable;
	}

	function enableBuildGrid(enable:Bool)
	{
		buildGrid.visible = enable;
	}

	function gameStateInit() // Event callback routine (EventManager subscribed)
	{
		var currentState:GameState = StateManager.getGameState();

		switch (currentState)
		{
			case None:
				return;
			case Welcome:
				welcomeWindow.activateWindow(true);
				enableHud(false);
			case About:
				aboutWindow.activateWindow(true);
				enableHud(false);
			case Select:
				buildWindow.activateWindow(true);
				enableHud(false);
			case Place:
				placeWindow.activateWindow(true);
				enableBuildGrid(true);
				enableHud(false);
				CursorManager.setCursorType(Add);
			case Remove:
				removeWindow.activateWindow(true);
				createRoomDeleteFrame();
				enableHud(false);
				CursorManager.setCursorType(Remove);
			case Game_Run:
				enableHud(true);
				CursorManager.setCursorType(Normal);
			case GameState.Win:
				winWindow.activateWindow(true);
				enableHud(false);
				CursorManager.setCursorType(Normal);
				SoundManager.playSound(ConfigManager.WIN_SOUND);
		}
	}

	function gameStateExit() // Event callback routine (EventManager subscribed)
	{
		var currentState:GameState = StateManager.getGameState();

		switch (currentState)
		{
			case None:
				return;
			case Welcome:
				welcomeWindow.activateWindow(false);
			case About:
				aboutWindow.activateWindow(false);
			case Select:
				buildWindow.activateWindow(false);
			case Place:
				placeWindow.activateWindow(false);
				enableBuildGrid(false);
				removeRoomBuildFrame();
			case Remove:
				removeWindow.activateWindow(false);
				removeRoomDeleteFrame();
			case Game_Run:
			case Win:
				winWindow.activateWindow(false);
		}
	}

	function allInvadersRemoved() // Event callback routine (EventManager subscribed)
	{
		StateManager.setGameState(Win);
	}

	// Create a build room frame for the gui
	public function createRoomBuildFrame(roomType:RoomType)
	{
		buildRoomFrame = new BuildRoomFrame(roomType);
		add(buildRoomFrame);
	}

	// Remove the build room frame from the gui
	public function removeRoomBuildFrame()
	{
		if (buildRoomFrame == null)
			return;

		remove(buildRoomFrame);
		buildRoomFrame = null;
	}

	// Create a delete room frame for the gui
	public function createRoomDeleteFrame()
	{
		deleteRoomFrame = new DeleteRoomFrame();
		add(deleteRoomFrame);
	}

	// Remove the delete room frame from the gui
	public function removeRoomDeleteFrame()
	{
		if (deleteRoomFrame == null)
			return;

		remove(deleteRoomFrame);
		deleteRoomFrame = null;
	}

	function updateDebugInfoLine(elapsed:Float)
	{
		FpsCalculator.update(elapsed);

		var mapIndex = MapManager.getMapIndexFromScreenPosXY(FlxG.mouse.x, FlxG.mouse.y);
		var isValid = MapManager.isValidMapIndex(mapIndex);

		text_debug.text = "Mouse: " + Std.string(FlxG.mouse.x) + "/" + Std.string(FlxG.mouse.y) + "  Map: " + Std.string(mapIndex.getX()) + "," + Std.string(mapIndex.getY()) + "/"
			+ Std.string(isValid) + "  " + FpsCalculator.getFpsString() + "  State: " + Std.string(StateManager.getGameState());
	}
}
