package classes;

import flixel.group.FlxSpriteGroup;
import flixel.util.FlxColor;
import flixel.text.FlxText;
import managers.WarehouseManager;
import managers.RocketManager;
import managers.ConfigManager;
import enums.CannonState;

class Cannon extends FlxSpriteGroup
{
	static final XLIM_LEFT = 40;
	static final XLIM_RIGHT = 480 - 40 - 64;
	static final CANNON_MAX_LVL = 9;
	static final CANNON_SPEED_MIN = 10.0;
	static final CANNON_SPEED_MAX = 90.0;

	var cannonLevel(default, null):Int = 0;
	var cannonState:CannonState = None;
	var cannonSprite:AtlasSprite;
	var cannonBaseSprite:AtlasSprite;
	var cannonLvlText:FlxText;
	var cannonSpeed = 0.0;
	var targetPosX = 0.0;

	public function new()
	{
		super();

		cannonSprite = new AtlasSprite();
		cannonSprite.assignAtlas();

		cannonSprite.animation.addByIndices(Std.string(CannonState.Waiting), ConfigManager.CANNON_GFX, [3], ".png", 0, false);
		cannonSprite.animation.addByIndices(Std.string(CannonState.Moving), ConfigManager.CANNON_GFX, [0,1,2], ".png", 3, true);
		cannonSprite.animation.addByIndices(Std.string(CannonState.Shooting), ConfigManager.CANNON_GFX, [3, 4, 5, 6, 3], ".png", 5, false);
		
		cannonSprite.animation.play(Std.string(CannonState.Waiting)); // needed so that resetSizeFromFrame works
		cannonSprite.resetSizeFromFrame();
		cannonSprite.centerOrigin();
		add(cannonSprite);		

		cannonBaseSprite = new AtlasSprite();
		cannonBaseSprite.assignAtlas();
		cannonBaseSprite.animation.addByIndices(Std.string(CannonState.Waiting), ConfigManager.CANNON_BASE_GFX, [4], ".png", 0, false);
		cannonBaseSprite.animation.addByIndices(Std.string(CannonState.Moving), ConfigManager.CANNON_BASE_GFX, [0, 1, 2, 3], ".png", 4, true);
		cannonBaseSprite.animation.addByIndices(Std.string(CannonState.Shooting), ConfigManager.CANNON_BASE_GFX, [4], ".png", 0, false);

		cannonBaseSprite.animation.play(Std.string(CannonState.Waiting)); // needed so that resetSizeFromFrame works
		cannonBaseSprite.resetSizeFromFrame();
		cannonBaseSprite.centerOrigin();
		cannonBaseSprite.setPosition(0, 64);
		add(cannonBaseSprite);			
		
		cannonLvlText = new FlxText(1, 29, 64, "0");
		cannonLvlText.setFormat(ConfigManager.STDFONT_NORMAL, 16, FlxColor.BLACK);
		cannonLvlText.alignment = CENTER;
		add(cannonLvlText);
	}

	public function setState(newState:CannonState)
	{
		if (cannonState == newState)
			return;

		if (newState != None)
		{
			cannonSprite.animation.play(Std.string(newState));
			cannonBaseSprite.animation.play(Std.string(newState));
		}

		cannonState = newState;
	}

	public function setCannonLevel(level:Int)
	{
		final speedStep = Math.round((CANNON_SPEED_MAX - CANNON_SPEED_MIN) / CANNON_MAX_LVL);

		cannonLevel = level < 0 ? 0 : level;
		cannonLevel = level > CANNON_MAX_LVL ? CANNON_MAX_LVL : level;

		cannonSpeed = CANNON_SPEED_MIN + (cannonLevel * speedStep);
		cannonLvlText.text = Std.string(cannonLevel);
		cannonBaseSprite.animation.getByName(Std.string(CannonState.Moving)).frameRate = 4 + cannonLevel;
	}

	function moveTo(targetPosX:Float)
	{
		if (targetPosX - x > 0)
			cannonBaseSprite.flipX = true;
		else 
			cannonBaseSprite.flipX = false;	
		
		this.targetPosX = targetPosX;
		setState(Moving);
	}

	public override function update(elapsed:Float)
	{
		super.update(elapsed);

		switch (cannonState)
		{
			case None:
				setState(Waiting);
			case Waiting:
				checkMovingStart();
			case Moving:
				handleMovement(elapsed);
			case Shooting:
				if (cannonSprite.animation.finished)
				{
					setState(Waiting);
				}
		}
	}

	function checkMovingStart()
	{
		if (WarehouseManager.getWareCount(Rockets) > 0) // We can move and shoot
		{			
			var newPosX = Std.random(XLIM_RIGHT - XLIM_LEFT) + XLIM_LEFT;
			moveTo(newPosX);
		}
	}

	function handleMovement(elapsed:Float)
	{
		var deltaPos = targetPosX - x;
		var deltaPosAbs = Math.abs(deltaPos);
		var moveDist = elapsed * cannonSpeed;

		if (deltaPosAbs <= moveDist)
		{
			x = targetPosX;
			RocketManager.startRocket(x + 28);
			WarehouseManager.addWareCount(Rockets, -1);
			setState(Shooting);
		}
		else
		{
			x += (deltaPos / deltaPosAbs * moveDist);
		}
	}
}
