package classes;

import enums.ButtonState;
import enums.CursorType;
import flixel.FlxG;
import flixel.FlxSprite;
import flixel.group.FlxSpriteGroup;
import flixel.text.FlxText;
import flixel.util.FlxColor;
import managers.ConfigManager;
import managers.CursorManager;
import managers.SoundManager;
import managers.SpriteManager;

/**
	My own implementation of a button class, based on the code of FlxButton.hx
	Features:
		3 operational states: Normal, MouseOver, Pressed  
		1 custom size bitmap per operational state (= 3) 
		remember cursor and set back after end of hovering
		bitmaps can be submitted by prefix name and will be retrieved from given atlas 
		Optionally, show a custom set font text on the button 
		Optionally, show a simple tooltip text below the button on mouseOver
		Optionally, play a custom set sound on button pressed 
 */
class TTButton extends FlxSpriteGroup
{
	public static inline final FONTSIZE:Int = 18;
	public static inline final FONTCOLOR:FlxColor = FlxColor.BLACK;

	var state:ButtonState = None;
	var oldCursor:CursorType = None;
	var buttonSprite:FlxSprite;
	var buttonLabel:FlxText;
	var buttonTooltip:FlxText;
	var pressCallback:Void->Void;

	public function new(spriteName:String, callback:Void->Void, buttonText:String = "", tooltipText:String = "")
	{
		super();

		buttonSprite = new FlxSprite();
		buttonSprite.frames = SpriteManager.getAtlas();
		assignSprites(spriteName);
		add(buttonSprite);

		if (buttonText.length > 0)
		{
			buttonLabel = new FlxText(0, 2, buttonSprite.width, buttonText);
			buttonLabel.setFormat(ConfigManager.STDFONT_BOLD, FONTSIZE, FONTCOLOR);
			buttonLabel.alignment = CENTER;
			add(buttonLabel);
		}

		if (tooltipText.length > 0)
		{
			buttonTooltip = new FlxText(0, buttonSprite.height, 0, tooltipText);
			buttonTooltip.setFormat(ConfigManager.STDFONT_NORMAL, FONTSIZE, FONTCOLOR);
			buttonTooltip.setPosition(buttonSprite.width / 2 - buttonTooltip.width / 2, buttonSprite.height - 4);
			add(buttonTooltip);
		}

		pressCallback = callback;
		setState(Normal);
	}

	public function assignSprites(spriteName:String)
	{
		buttonSprite.animation.destroyAnimations();
		buttonSprite.animation.addByIndices(Std.string(ButtonState.Normal), spriteName, [0], ".png", 0, false);
		buttonSprite.animation.addByIndices(Std.string(ButtonState.Highlighted), spriteName, [1], ".png", 0, false);
		buttonSprite.animation.addByIndices(Std.string(ButtonState.Pressed), spriteName, [2], ".png", 0, false);
		buttonSprite.animation.play(Std.string(ButtonState.Normal)); // Needed to get all the internal sizes updated.
		buttonSprite.resetSizeFromFrame();
		buttonSprite.centerOrigin();
		buttonSprite.updateHitbox();
	}

	function setState(newState:ButtonState)
	{
		if (state == newState)
			return;

		switch (newState)
		{
			case None:
			case Normal:
				buttonSprite.animation.play(Std.string(newState));
				if (oldCursor != None)
				{
					CursorManager.setCursorType(oldCursor);
					oldCursor = None;
				}

				if (buttonTooltip != null)
					buttonTooltip.visible = false;
			case Highlighted:
				buttonSprite.animation.play(Std.string(newState));
				oldCursor = CursorManager.getCurrentCursorType();
				CursorManager.setCursorType(Link);
				if (buttonTooltip != null)
					buttonTooltip.visible = true;
			case Pressed:
				SoundManager.playSound(ConfigManager.BUTTONCLICK_SOUND);
				CursorManager.setCursorType(Link);
				buttonSprite.animation.play(Std.string(newState));
				if (buttonTooltip != null)
					buttonTooltip.visible = false;
		}

		state = newState;
	}

	override public function update(elapsed:Float)
	{
		super.update(elapsed);

		if (FlxG.mouse.overlaps(buttonSprite))
		{
			if (FlxG.mouse.justPressed)
			{
				setState(ButtonState.Pressed);
			}
			else if (FlxG.mouse.justReleased && state == ButtonState.Pressed)
			{
				setState(ButtonState.Normal);
				pressCallback();
			}
			else if (state != ButtonState.Pressed)
			{
				setState(ButtonState.Highlighted);
			}
		}
		else
		{
			setState(ButtonState.Normal);
		}
	}
}
