package classes;

import enums.Direction;
import enums.WorkerState;
import helpers.MapIndex;
import lime.math.Vector2;
import managers.ConfigManager;
import managers.MapManager;
import managers.PathManager;

class Worker extends AtlasSprite
{
	static final WORKER_SPEED = 20.0;

	var pathFinder:PathFinder = new PathFinder();
	var workerState:WorkerState = None;
	var walkPath:Array<MapIndex>;
	var targetPos:Vector2;

	public function new()
	{
		super();
		assignAtlas();
		animation.addByIndices(Std.string(WorkerState.Waiting), ConfigManager.WORKER_GFX, [0], ".png", 0, false);
		animation.addByIndices(Std.string(WorkerState.WalkingLeftRight), ConfigManager.WORKER_GFX, [1, 2, 3, 4], ".png", 6, true);
		animation.addByIndices(Std.string(WorkerState.WalkingUpDown), ConfigManager.WORKER_GFX, [5, 6], ".png", 4, true);
		animation.play(Std.string(WorkerState.Waiting)); // needed so that resetSizeFromFrame works
		resetSizeFromFrame();
		centerOrigin();

		walkPath = new Array<MapIndex>();
		pathFinder = new PathFinder();
	}

	public function setState(newState:WorkerState)
	{
		if (workerState == newState)
			return;

		if (newState != None)
			animation.play(Std.string(newState));

		workerState = newState;
	}

	function setStateFromWalkingDirection(walkingDir:Direction)
	{
		switch (walkingDir)
		{
			case None:
				setState(Waiting);
			case Left:
				setState(WalkingLeftRight);
				this.flipX = true;
			case Right:
				setState(WalkingLeftRight);
				this.flipX = false;
			case Up | Down:
				setState(WalkingUpDown);
		}
	}

	public function setTargetPos(mapIndex:MapIndex)
	{
		targetPos = MapManager.getScreenXYfromMapIndex(mapIndex);
		targetPos.x += 2;
		targetPos.y += 2;
	}

	function setTargetPath(path:Array<MapIndex>)
	{
		walkPath = path;
	}

	public override function update(elapsed:Float)
	{
		super.update(elapsed);

		switch (workerState)
		{
			case None:
				setState(Waiting);
			case Waiting:
				setNextMovement();
			case WalkingLeftRight:
				handleMovement(elapsed);
			case WalkingUpDown:
				handleMovement(elapsed);
		}
	}

	function handleMovement(elapsed:Float)
	{
		var deltaPos = targetPos.subtract(new Vector2(x, y));
		var deltaPosAbs = deltaPos.length;
		var moveDist = elapsed * WORKER_SPEED;

		if (deltaPosAbs <= moveDist)
		{
			x = targetPos.x;
			y = targetPos.y;
			setNextMovement();
		}
		else
		{
			x += (deltaPos.x / deltaPosAbs * moveDist);
			y += (deltaPos.y / deltaPosAbs * moveDist);
		}
	}

	function setNextMovement()
	{
		var currentTile = MapManager.getMapIndexFromScreenPosXY(x, y);

		if (walkPath.length > 0) // Still waypoints in the current path
		{
			var nextWP = walkPath.shift();
			setTargetPos(nextWP);
			setStateFromWalkingDirection(currentTile.getDirectionOf(nextWP));
		}
		else
		{
			if (pathFinder.state == Running) // Path calculation under way
			{
				if (pathFinder.update() == Finished) // Calculation has just finished
				{
					setTargetPath(pathFinder.path);
					// On next update(), this will trigger a state change
				}
			}
			else // Trigger new path calculation
			{
				var newTarget = PathManager.getRandomPathPosition();
				pathFinder.startPathCalc(currentTile, newTarget);
				setState(Waiting);
			}
		}
	}
}
