package managers;

import enums.RoomType;
import helpers.MapIndex;
import lime.math.Vector2;

class MapManager
{
	// Map origin is in the upper left corner of the usable building space
	public static final MAPSIZE_X:Int = 10;
	public static final MAPSIZE_Y:Int = 8;
	public static final TILESIZE_X:Int = 40;
	public static final TILESIZE_Y:Int = 40;

	static final MAP_PX_ORIGIN = new Vector2(40, 440);

	// A 2-dim array to represent room ID per MapIndex tile
	static final map_array:Array<Array<Int>> = [
		for (x in 0...MAPSIZE_X) [
			for (y in 0...MAPSIZE_Y) 0
		]
	];

	public static function getMapIndexFromScreenPosV2(v2:Vector2):MapIndex
	{
		return getMapIndexFromScreenPosXY(v2.x, v2.y);
	}

	public static function getMapIndexFromScreenPosXY(x:Float, y:Float):MapIndex
	{
		var x_ret:Int = Math.floor((x - MAP_PX_ORIGIN.x) / TILESIZE_X);
		var y_ret:Int = Math.floor((y - MAP_PX_ORIGIN.y) / TILESIZE_Y);
		return new MapIndex(x_ret, y_ret);
	}

	public static function getScreenXYfromMapIndex(mapIndex:MapIndex):Vector2
	{
		return getScreenXYfromMapIndexXY(mapIndex.getX(), mapIndex.getY());
	}

	public static function getScreenXYfromMapIndexXY(x:Int, y:Int):Vector2
	{
		return new Vector2(MAP_PX_ORIGIN.x + x * TILESIZE_X, MAP_PX_ORIGIN.y + y * TILESIZE_Y);
	}

	public static function isValidMapIndex(mapIndex:MapIndex)
	{
		return isValidMapIndexXY(mapIndex.getX(), mapIndex.getY());
	}

	public static function isValidMapIndexXY(x:Int, y:Int):Bool
	{
		if (x < 0 || y < 0)
			return false;
		if (x >= MAPSIZE_X || y >= MAPSIZE_Y)
			return false;

		return true;
	}

	public static function canBuildHere(mapIndex:MapIndex, roomType:RoomType):Bool
	{
		return canBuildHereXY(mapIndex.getX(), mapIndex.getY(), roomType);
	}

	public static function canBuildHereXY(map_x:Int, map_y:Int, roomType:RoomType):Bool
	{
		var x_size = RoomManager.getRoomSizeXfromKind(roomType);
		for (x in map_x...(map_x + x_size))
		{
			// Check if any future room tile is invalid
			if (isValidMapIndexXY(x, map_y) == false)
				return false;

			// Check if any future room tile is occupied right now
			if (isOccupiedXY(x, map_y) == true)
				return false;
		}

		// Yeah, can build
		return true;
	}

	public static function setRoomToTile(mapIndex:MapIndex, roomId:Int)
	{
		setRoomToTileXY(mapIndex.getX(), mapIndex.getY(), roomId);
	}

	public static function setRoomToTileXY(map_X:Int, map_Y:Int, roomId:Int)
	{
		if (!isValidMapIndexXY(map_X, map_Y))
			throw "setRoomToTileXY: Map index not valid!";
		map_array[map_X][map_Y] = roomId;
	}

	public static function isOccupied(mapIndex:MapIndex):Bool
	{
		return isOccupiedXY(mapIndex.getX(), mapIndex.getY());
	}

	public static function isOccupiedXY(map_x:Int, map_y:Int):Bool
	{
		return map_array[map_x][map_y] != 0;
	}

	public static function getRoomIdFromTile(mapIndex:MapIndex):Int
	{
		return getRoomIdFromTileXY(mapIndex.getX(), mapIndex.getY());
	}

	public static function getRoomIdFromTileXY(map_x:Int, map_y:Int):Int
	{
		if (!isValidMapIndexXY(map_x, map_y))
			return 0; // Invlid is always empty
		return map_array[map_x][map_y]; // Will return '0' if empty (no room)
	}

	public static function setMapTilesToRoom(firstMapIndex:MapIndex, roomType:RoomType, roomId:Int)
	{
		setMapTilesToRoomXY(firstMapIndex.getX(), firstMapIndex.getY(), roomType, roomId);
	}

	public static function setMapTilesToRoomXY(firstMap_x:Int, firstMap_y:Int, roomType:RoomType, roomId:Int)
	{
		var x_size = RoomManager.getRoomSizeXfromKind(roomType);

		for (x in firstMap_x...(firstMap_x + x_size))
		{
			setRoomToTileXY(x, firstMap_y, roomId);
		}
	}

	public static function getFirstRoomIndex(roomId:Int):MapIndex
	{
		for (y in 0...MAPSIZE_Y)
			for (x in 0...MAPSIZE_X)
			{
				if (map_array[x][y] == roomId)
				{
					return (new MapIndex(x, y));
				}
			}

		return null;
	}

	public static function getRoomIndices(roomId:Int):Array<MapIndex>
	{
		var retArray = new Array<MapIndex>();

		for (y in 0...MAPSIZE_Y)
			for (x in 0...MAPSIZE_X)
			{
				if (map_array[x][y] == roomId)
				{
					retArray.push(new MapIndex(x, y));
				}
			}

		return retArray; // This should return indices in right order (left->right)
	}

	public static function getRandomMapTile():MapIndex
	{
		var x = Std.random(MAPSIZE_X);
		var y = Std.random(MAPSIZE_Y);
		return new MapIndex(x, y);
	}
}
