package managers;

import enums.Direction;
import helpers.Array2D;
import helpers.MapIndex;

enum PathDirection
{
	x; // not walkable
	a; // up down
	b; // up down right
	c; // up down left
	d; // up right
	e; // down right
	f; // up left
	g; // down left
	h; // left right
}

/*
	The possible movements between tiles and assignment of map/path indices
	.. XY   0  1  2  3  4  5  6  7  8  9 10 11 local pathfinding
	XY XY  -1  0  1  2  3  4  5  6  7  8  9 10 real map
	0  -2   e  h  h  h  h  h  h  h  h  h  h  g
	1  -1   a  x  x  x  x  x  x  x  x  x  x  a
	2  +0   b  h  h  h  h  h  h  h  h  h  h  c
	3  +1   b  h  h  h  h  h  h  h  h  h  h  c
	4  +2   b  h  h  h  h  h  h  h  h  h  h  c
	5  +3   b  h  h  h  h  h  h  h  h  h  h  c
	6  +4   b  h  h  h  h  h  h  h  h  h  h  c
	7  +5   b  h  h  h  h  h  h  h  h  h  h  c
	8  +6   b  h  h  h  h  h  h  h  h  h  h  c
	9  +7   d  h  h  h  h  h  h  h  h  h  h  c
    l   r
 */

 // This class is providing general helper functions for pathfinding.
 // Actual work is done "threaded-like" in PathFinder class (one per Worker)
class PathManager
{
	public static final PATHMAP:Array2D<PathDirection> = new Array2D<PathDirection>(12, 10, [
		e, h, h, h, h, h, h, h, h, h, h, g,
		a, x, x, x, x, x, x, x, x, x, x, a,
		b, h, h, h, h, h, h, h, h, h, h, c,
		b, h, h, h, h, h, h, h, h, h, h, c,
		b, h, h, h, h, h, h, h, h, h, h, c,
		b, h, h, h, h, h, h, h, h, h, h, c,
		b, h, h, h, h, h, h, h, h, h, h, c,
		b, h, h, h, h, h, h, h, h, h, h, c,
		b, h, h, h, h, h, h, h, h, h, h, c,
		d, h, h, h, h, h, h, h, h, h, h, f
	]);

	public static final PATHDIRS:Map<PathDirection, Array<Direction>> = [
		x => [],
		a => [Up, Down],
		b => [Up, Down, Right],
		c => [Up, Down, Left],
		d => [Up, Right],
		e => [Down, Right],
		f => [Up, Left],
		g => [Down, Left],
		h => [Left, Right]
	];

	public static function mapIndexToPathIndex(mi:MapIndex):MapIndex
	{
		return new MapIndex(mi.getX() + 1, mi.getY() + 2);
	}

	public static function pathIndexToMapIndex(pi:MapIndex):MapIndex
	{
		return new MapIndex(pi.getX() - 1, pi.getY() - 2);
	}

	// Which directions can be walked to from this tile?
	public static function getMovement(pi:MapIndex):Array<Direction>
	{		
		return PATHDIRS[PATHMAP.get2D(pi.getX(), pi.getY())];
	}

	// Get the adjacent tiles that can be walked on, based on the
	// PathManager's stored 'allowed movement' directions
	public static function getAdjacendWalkable(pi:MapIndex):Array<MapIndex>
	{
		var directions = getMovement(pi); // get the directions we can move to
		var targets = directions.map(f -> pi.getInDirection(f)); // get the tiles we can move to
		return targets;
	}

	// get an allowed worker position on the map, using PATHMAP entries
	public static function getRandomPathPosition():MapIndex
	{
		var pi:MapIndex;
		do
			pi = new MapIndex(Std.random(12), Std.random(10)) // 0..11, 0..9
		while 
			(getMovement(pi).length == 0);

		return pathIndexToMapIndex(pi);
	}
}
